part of simple_billing;

class _SimpleBillingData {
  static Box _purchaseBox;
  static Box _subscriptionIAPItemsBox;
  static Box _productIAPItemsBox;
  static const String _boxPrefix = 'box_simple_billing';

  static Future<void> init() async {
    final Directory appDocumentDir =
        await path_provider.getApplicationDocumentsDirectory();
    Hive.initFlutter(appDocumentDir.path);
    _purchaseBox = await Hive.openBox('${_boxPrefix}_purchase');
    _subscriptionIAPItemsBox =
        await Hive.openBox('${_boxPrefix}_subscriptionIAPItems');
    _productIAPItemsBox = await Hive.openBox('${_boxPrefix}_productIAPItems');
  }

  static Set<PurchasedItem> get products {
    final Set<PurchasedItem> items = {};
    for (final product in _purchaseBox.values) {
      items.add(
        PurchasedItem.fromJSON(
            json.decode(product as String) as Map<String, dynamic>),
      );
    }
    return items;
  }

  static Future<void> saveProducts(Set<PurchasedItem> products) async {
    await _purchaseBox.clear();
    final Set<String> usedIds = {};
    for (final product in products) {
      if (usedIds.add(product.productId)) {
        await _purchaseBox.put(
            product.productId, json.encode(_productToJson(product)));
      }
    }
  }

  static List<IAPItem> getIAPItems({bool isSubscription = false}) {
    final Box box =
        isSubscription ? _subscriptionIAPItemsBox : _productIAPItemsBox;
    return List<Map<String, dynamic>>.from(box.values)
        .map((e) => IAPItem.fromJSON(e))
        .toList();
  }

  static Future<void> saveIAPItems(List<IAPItem> items,
      {bool isSubscription = false}) async {
    final Box box =
        isSubscription ? _subscriptionIAPItemsBox : _productIAPItemsBox;
    await box.clear();
    for (final item in items) {
      await box.put(item.productId, _iAPItemToJson(item));
    }
  }

  static Map<String, dynamic> _productToJson(PurchasedItem product) {
    return {
      'productId': product.productId,
      'transactionId': product.transactionId,
      'transactionDate': product.transactionDate?.millisecondsSinceEpoch,
      'transactionReceipt': product.transactionReceipt,
      'purchaseToken': product.purchaseToken,
      'orderId': product.orderId,

      /// android specific
      'dataAndroid': product.dataAndroid,
      'signatureAndroid': product.signatureAndroid,
      'isAcknowledgedAndroid': product.isAcknowledgedAndroid,
      'autoRenewingAndroid': product.autoRenewingAndroid,
      'purchaseStateAndroid': product.purchaseStateAndroid,
      'developerPayloadAndroid': product.developerPayloadAndroid,
      'originalJsonAndroid': product.originalJsonAndroid,

      /// ios specific
      'originalTransactionDateIOS':
          product.originalTransactionDateIOS?.millisecondsSinceEpoch,
      'originalTransactionIdentifierIOS':
          product.originalTransactionIdentifierIOS,
      'transactionStateIOS': product.transactionStateIOS?.index,
    };
  }

  static Map<String, dynamic> _iAPItemToJson(IAPItem item) {
    return {
      'productId': item.productId,
      'price': item.price,
      'currency': item.currency,
      'localizedPrice': item.localizedPrice,
      'title': item.title,
      'description': item.description,
      'introductoryPrice': item.introductoryPrice,
      'subscriptionPeriodNumberIOS': item.subscriptionPeriodNumberIOS,
      'subscriptionPeriodUnitIOS': item.subscriptionPeriodUnitIOS,
      'introductoryPricePaymentModeIOS': item.introductoryPricePaymentModeIOS,
      'introductoryPriceNumberOfPeriodsIOS':
          item.introductoryPriceNumberOfPeriodsIOS,
      'introductoryPriceSubscriptionPeriodIOS':
          item.introductoryPriceSubscriptionPeriodIOS,
      'subscriptionPeriodAndroid': item.subscriptionPeriodAndroid,
      'introductoryPriceCyclesAndroid': item.introductoryPriceCyclesAndroid,
      'introductoryPricePeriodAndroid': item.introductoryPricePeriodAndroid,
      'freeTrialPeriodAndroid': item.freeTrialPeriodAndroid,
      'iconUrl': item.iconUrl,
      'originalJson': item.originalJson,
      'originalPrice': item.originalPrice,
    };
  }
}
